﻿package MT::Plugin::MeCab;

use strict;
use MT;
use MT::Plugin;

our $VERSION = '0.2';

use base qw( MT::Plugin );

@MT::Plugin::MeCab::ISA = qw( MT::Plugin );

my $plugin = __PACKAGE__->new( {
    id  => 'MeCab',
    key => 'MeCab',
    description => 'MeCab example',
    name => 'MeCab',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    version => $VERSION,
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        callbacks => {
            'cms_post_save.entry'
                => \&_cms_post_save_entry,
            'cms_post_save.page'
                => \&_cms_post_save_entry,
        },
        tags => {
            modifier => {
                'kanji2kana' => \&_kanji2kana,
            },
        },
    } );
}

sub _kanji2kana {
    my( $str, $arg, $ctx ) = @_;
    eval { require MeCab };
    unless ( $@ ){
        my $mecab = MeCab::Tagger->new();
        my $node = $mecab->parseToNode( $str );
        my @words;
        for ( ; $node; $node = $node->{ next } ) {
            next unless defined $node->{ surface };
            my $parse = $node->{ feature };
            push ( @words, $parse );
        }
        my %count;
        @words = grep { !$count{ $_ }++ } @words;
        for my $parse ( @words ) {
            my @item = split ( /,/, $parse );
            my $search = quotemeta ( $item[6] );
            my $replace = $item[7];
            $str =~ s/$search/$replace/g;
        }
        return $str;
    } else {
        MT->log( 'MeCab is not available' );
        return $str;
    }
}

sub _cms_post_save_entry {
    my ( $cb, $app, $obj, $original ) = @_;
    eval { require MeCab };
    unless ( $@ ){
        use Encode qw( decode );
        my $str = $obj->title . ' ' . $obj->title . ' ' . $obj->text;
        $str =~ s/<script.{1,}?<\/script>//sg;
        $str =~ s/<!\-\-[^>]*?\-\->//sg;
        $str =~ s/<img[^>]*?alt=*["']([^"'>]*)["'][^>]*?>/$1/gi;
        $str =~ s/<[^>]*?>//sg;
        $str =~ s/[\n|\t]//sg;
        $str =~ s/"//sg;
        my $mecab = MeCab::Tagger->new();
        my $node = $mecab->parseToNode( $str );
        my @tags; # = $obj->tags;
        unless ( defined $original ) {
            @tags = $obj->tags;
        }
        my @words;
        for ( ; $node; $node = $node->{ next } ) {
            next unless defined $node->{ surface };
            # my $surface = $node->{ surface };
            my $parse = $node->{ feature };
            # $app->log( $parse );
            if ( ( $parse =~ /^名詞,一般,/ ) ||
                 ( $parse =~ /^名詞,固有名詞,組織,/ ) ) {
                my @item = split ( /,/, $parse );
                my $word = $item[6];
                next if ( $word =~ /会社$/ );
                my $length;
                if ( MT->version_number >= 5 ) {
                    $length = length( $word );
                } else {
                    $length = length( decode( 'utf-8', $word ) );
                }
                if ( $parse =~ /Tags$/ ) {
                    push ( @tags, $word );
                } else {
                    if ( ( $length > 7 ) && ( $word ne '*' ) ) {
                        push ( @words, $word );
                    }
                }
            }
        }
        my %counter;
        for my $key ( @words ) {
            $counter{ $key } = $counter{ $key } + 1;
        }
        for my $key ( sort{ $counter{ $b } cmp $counter{ $a } } keys %counter ) {
            if ( $counter{ $key } > 1 ) {
                push ( @tags, '@' . $key );
            } else {
                last;
            }
        }
        $obj->set_tags ( @tags ) if ( @tags );
        $obj->save or die $obj->errstr;
    } else {
        $app->log( 'MeCab is not available' );
    }
}

1;